from typing import Dict
from workspace import env, message, git
import os
from colorama import Fore, Back, Style

toplevel_cmake = """
cmake_minimum_required(VERSION 3.16.3)
project(wks)

add_subdirectory(src)
"""

def generate():
  message.bright('* Generating CMake')
  print('')

  cmake = "cmake_minimum_required(VERSION 3.16.3)\n\n"

  for directory in git.get_directories():
    cmakes = ['']

    config = env.get_config(directory)
    if config and 'cmakes' in config:
      cmakes = config['cmakes']

    for cmake_name in cmakes:
      cmake_directory = os.path.realpath(directory)
      dname = os.path.basename(os.path.dirname(directory))
      name = os.path.basename(directory)
      project = '%s_%s' % (dname, name)

      if cmake_name:
        project += '_' + cmake_name
        cmake_directory += '/' + cmake_name

      cmake += "add_subdirectory(%s %s)\n" % (cmake_directory, project)
      print('- %s [%s]' % (project, Style.DIM + cmake_directory + Style.RESET_ALL))

  f = open(env.sources_directory + '/CMakeLists.txt', 'w')
  f.write(cmake)
  f.close()

  if not os.path.exists('CMakeLists.txt'):
      message.bright("* No top-level CMakeLists.txt found, generating one...")
      f = open('CMakeLists.txt', 'w')
      f.write(toplevel_cmake)
      f.close()

  message.bright("* Wrote CMakeLists.txt")
  print('')



import os
from workspace import env, git, message, cmake

name = 'install'
help = """Installs a module given its name"""
usage = "[repository [repository [...]]]"
min_args = 0

def run(args):
  for repository in args:
    if not git.install(repository):
      message.error("Not installing "+repository+" because the directory already exists")
  git.scan_all_dependencies()
  cmake.generate()
  
  



import os
import yaml

sources_directory = os.getcwd() + '/src'

def get_config(directory):
  deps_filename = directory + '/wks.yml'
  if os.path.exists(deps_filename):
    f = open(deps_filename, 'r')
    deps = yaml.load(f, Loader=yaml.Loader)

    return deps
    
  return None


import os
from colorama import Fore, Back, Style

def error(message):
  print(Style.BRIGHT + Fore.RED + message + Style.RESET_ALL)

def bright(message):
  print('')
  print(Style.BRIGHT + message + Style.RESET_ALL)

def run_or_fail(cmd):
  if os.system(cmd) != 0:
    die('Error while running '+cmd)

def emphasis(text):
  return Fore.BLUE + text + Fore.RESET

def warn(text):
  return Fore.YELLOW + text + Fore.RESET

def error(text):
  return Fore.RED + text + Fore.RESET

def success(text):
  return Fore.GREEN + text + Fore.RESET

def die(message):
  print(error(message))
  exit()


import sys

from workspace import command_build, command_cmake, command_help, command_install, command_pull, command_status

commands = [command_build, command_cmake, command_help, command_install, command_pull, command_status]



import os
from workspace import env, git, message, cmake

name = 'pull'
help = """Pull all the repositories"""
usage = ""
min_args = 0

def run(args):
  git.global_command('git pull')
  git.scan_all_dependencies()
  cmake.generate()
  


from colorama import Fore, Back, Style
from workspace import commands, message

name = "help"
help = """Provides help about commands"""
usage = ""
min_args = 0

def run(args):
  print('Available commands:')
  print('')

  for entry in commands.commands:
    message.bright(message.emphasis("wks") + " " + entry.name + ' ' + Style.RESET_ALL + entry.usage)
    lines = entry.help.strip().split("\n")
    lines = ["    " + line for line in lines]
    print("\n".join(lines))

  print('')


import yaml
import subprocess
import os
from workspace import env, message, env

def parse_repository_name(name):
  parts = name.split('/')

  if len(parts) != 2:
    message.die('Bad repository name: ' + name)

  repository = {
    'vendor': parts[0],
    'name': parts[1],
    'branch': None,
    'tag': None
  }

  markers = {'#': 'branch', '@': 'tag'}

  for marker in markers:
    if marker in repository['name']:
      name, value = repository['name'].split(marker, 1)
      repository['name'] = name
      repository[markers[marker]] = value

  repository['directory'] = env.sources_directory + '/' + repository['vendor'] + '/' + repository['name']
  repository['git'] = 'git@github.com:%s/%s.git' % (repository['vendor'], repository['name'])
  repository['full_name'] = '%s/%s' % (repository['vendor'], repository['name'])

  return repository

def install(repository_name):
  repository = parse_repository_name(repository_name)
  
  if not os.path.exists(repository['directory']):
    message.bright('* Installing ' + repository_name + '...')
    vendor_dir = os.path.dirname(repository['directory'])

    if not os.path.exists(vendor_dir):
      os.makedirs(vendor_dir)
    
    branch = ''
    if repository['branch']:
      branch = '--branch '+repository['branch']
    if repository['tag']:
      branch = '--branch '+repository['tag']

    cmd = 'git clone %s %s %s' % (branch, repository['git'], repository['directory'])

    message.run_or_fail(cmd)

    return True
  else:
    return False

def get_directories(directory=None):
  directories = []
  if directory is None:
    directory = env.sources_directory

  if not os.path.isdir(directory):
    message.die("Directory %s does not exists" % directory)
  
  for entry in os.scandir(directory):
    full_name = directory + '/' + entry.name
    if os.path.isdir(full_name):
      if os.path.isdir(full_name+'/.git'):
        directories.append(full_name)
      else:
        directories += get_directories(full_name)

  return directories

def scan_dependencies(directory):
  changed = False
  config = env.get_config(directory)
  
  if config and 'deps' in config:
    for entry in config['deps']:
      if install(entry):
        changed = True

  return changed

def scan_all_dependencies():
  message.bright('* Scanning all dependencies')
  changed = True

  while changed:
    changed = False
    for directory in get_directories():
      if scan_dependencies(directory):
        changed = True

def global_command(command):
  message.bright("* Running global command: %s" % command)

  for directory in get_directories():
    message.bright('- In %s ...' % os.path.realpath(directory))
    cmd = 'cd %s; %s' % (directory, command)
    os.system(cmd)
  
  print('')

def status(directory):
  output = subprocess.getoutput("cd %s; git status --porcelain=v1" % directory)
  return output

def branch(directory):
  output = subprocess.getoutput("cd %s; git rev-parse --abbrev-ref HEAD" % directory)
  return output

def is_ahead(directory):
  ret, output = subprocess.getstatusoutput("cd %s; git rev-list @{u}.." % directory)
  return ret == 0 and output.strip() != ''